#!/bin/sh
ROOT=$(pwd)
src=$1
dst=$2
mode=_$3
BR=$scriptdir/br
UNBR=$scriptdir/unbr

tmpdir=$ROOT/tmp/tmp_migrate
special_case_list=$tmpdir/special_case_list
admin_writable_list=$tmpdir/admin_writable_list
dst_config_list=$tmpdir/dst_config_list
src_new_list=$tmpdir/src_new_list
tmp_file=$tmpdir/tmp_file

rm -rf $tmpdir
mkdir $tmpdir

## List here the files excluded from copy (need special processing)
if [ $mode == _safemode ]; then
 	echo "" | $BR > $special_case_list
 	echo "=== WARNING: New package has no rules for config migration."
	echo "=== Configuration copied in safe-mode, check after upgrade."
else
echo config/syslog.conf | $BR > $special_case_list
fi

## Copy files which admin-writable in dst (without permissions)
cd $dst
find . \( \( -perm +020 -group 1000 \) -o \( -perm +200 -user 1000 \) \) -type f \
| cut -c3- | $BR | grep -vf $special_case_list | $UNBR > $admin_writable_list
for filename in `cat $admin_writable_list`
do
	if [ -e $src/$filename ]; then cp $src/$filename $dst/$filename; echo Copy etc/$filename; fi
done

## Copy new files and dirs from src/config (with permissions)
cd $dst/config
find . | cut -c3- | grep -v '^$'| $BR > $dst_config_list
cd $src/config
find . | cut -c3- | grep -v '^$' | $BR | grep -vf $dst_config_list | $UNBR > $src_new_list
for filename in `cat $src_new_list`
do
	cp -pr $src/config/$filename $dst/config/$filename; echo Copy etc/config/$filename;
done

## Special cases:
# etc/shadow
echo "Copy etc/shadow"
cp $src/shadow $dst/

if [ $mode != _safemode ]; then
# etc/config/syslog.conf 
if 	[ -x $src/config/syslog.conf ]; then
	echo "Copy etc/config/syslog.conf"
	cp $src/config/syslog.conf $dst/config/
else
	if [ -f $src/config/syslog.conf ]; then
		echo "Changed format of etc/config/syslog.conf"
		echo 'syslogd' `cat $src/config/syslog.conf` > $tmp_file
		echo '## new defaults: ' `cat $dst/config/syslog.conf` >> $tmp_file
		cp $tmp_file $dst/config/syslog.conf
	fi
fi

# etc/HOSTNAME
if [ ! -f $src/HOSTNAME ]
then
	echo "Hostname moved from etc/config/netconfig.sh to etc/HOSTNAME"
	grep "hostname" $dst/config/netconfig.sh | cut -f2 -d" " > $dst/HOSTNAME
	grep -v "hostname" $dst/config/netconfig.sh > $tmp_file
	cp $tmp_file $dst/config/netconfig.sh
fi
fi
## cleanup
rm -rf $tmpdir
